class X2TextConstants:
    PLATFORM_SERVICE_API_KEY = "PLATFORM_SERVICE_API_KEY"
    X2TEXT_HOST = "X2TEXT_HOST"
    X2TEXT_PORT = "X2TEXT_PORT"
    ENABLE_HIGHLIGHT = "enable_highlight"
    TAGS = "tags"
    EXTRACTED_TEXT = "extracted_text"
    WHISPER_HASH = "whisper-hash"
    WHISPER_HASH_V2 = "whisper_hash"
