from .llama_parse import LlamaParseAdapter

metadata = {
    "name": LlamaParseAdapter.__name__,
    "version": "1.0.0",
    "adapter": LlamaParseAdapter,
    "description": "LlamaParse X2Text adapter",
    "is_active": True,
}
