from .google_document_ai import GoogleDocumentAI

metadata = {
    "name": GoogleDocumentAI.__name__,
    "version": "1.0.0",
    "adapter": GoogleDocumentAI,
    "description": "Google Document AI OCR adapter",
    "is_active": True,
}
