from .vertex_ai import VertexAIEmbedding

metadata = {
    "name": VertexAIEmbedding.__name__,
    "version": "1.0.0",
    "adapter": VertexAIEmbedding,
    "description": "VertexAI embedding adapter",
    "is_active": True,
}
