from .llm_whisperer_v2 import LLMWhispererV2

metadata = {
    "name": LLMWhispererV2.__name__,
    "version": "1.0.0",
    "adapter": LLMWhispererV2,
    "description": "LLMWhispererV2 X2Text adapter",
    "is_active": True,
}
