class LlamaParseConfig:
    """Dictionary keys used to process LlamaParse."""

    API_KEY = "api_key"
    BASE_URL = "base_url"
    RESULT_TYPE = "result_type"
    NUM_WORKERS = "num_workers"
    VERBOSE = "verbose"
    LANGUAGE = "language"
