from .no_op_embedding import NoOpEmbedding

metadata = {
    "name": NoOpEmbedding.__name__,
    "version": "1.0.0",
    "adapter": NoOpEmbedding,
    "description": "NoOp embedding adapter",
    "is_active": True,
}
