from .azure_open_ai import AzureOpenAI

metadata = {
    "name": AzureOpenAI.__name__,
    "version": "1.0.0",
    "adapter": AzureOpenAI,
    "description": "AzureOpenAI embedding adapter",
    "is_active": True,
}
