class FileType:
    TEXT_PLAIN = "text/plain"
    IMAGE_JPEG = "image/jpeg"
    IMAGE_PNG = "image/png"
    IMAGE_TIFF = "image/tiff"
    IMAGE_BMP = "image/bmp"
    IMAGE_GIF = "image/gif"
    IMAGE_WEBP = "image/webp"
    APPLICATION_PDF = "application/pdf"
    ALLOWED_TYPES = [
        IMAGE_JPEG,
        IMAGE_PNG,
        IMAGE_TIFF,
        IMAGE_BMP,
        IMAGE_GIF,
        IMAGE_WEBP,
        APPLICATION_PDF,
    ]
