from .llm_whisperer import LLMWhisperer

metadata = {
    "name": LLMWhisperer.__name__,
    "version": "1.0.0",
    "adapter": LLMWhisperer,
    "description": "LLMWhisperer X2Text adapter",
    "is_active": True,
}
