from .hugging_face import HuggingFace

metadata = {
    "name": HuggingFace.__name__,
    "version": "1.0.0",
    "adapter": HuggingFace,
    "description": "HuggingFace embedding adapter",
    "is_active": False,
}
