class Common:
    METADATA = "metadata"
    MODULE = "module"
    ADAPTER = "adapter"
    SRC_FOLDER = "src"
    ADAPTER_METADATA = "adapter_metadata"
    ICON = "icon"
    ADAPTER_ID = "adapter_id"
    ADAPTER_TYPE = "adapter_type"
    DEFAULT_ERR_MESSAGE = "Something went wrong"
